package textfileoutput;

import static org.junit.Assert.assertEquals;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;

import org.apache.poi.util.IOUtils;
import org.junit.After;
import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.TemporaryFolder;
import org.pentaho.di.core.KettleEnvironment;
import org.pentaho.di.core.logging.LogLevel;
import org.pentaho.di.trans.Trans;
import org.pentaho.di.trans.TransMeta;

public class TextFileOutputTest {

	@Rule
	public TemporaryFolder tempFolder = new TemporaryFolder();

	@Before
	public void setUp() throws Exception {
		KettleEnvironment.init();
	}

	@After
	public void tearDown() {
		KettleEnvironment.shutdown();
	}

	@Test
	public void testGeneratedInputCreateAtStart() throws Exception {
		TransMeta meta = createTransMeta("generated-input-create-at-start.ktr", "number_rows", "10");
		Trans trans = new Trans(meta);

		trans.execute(new String[0]);
		trans.waitUntilFinished();

		assertEquals("Expected one result file", 1, trans.getResult().getResultFilesList().size());
		System.out.println(trans.getResult().getResultFilesList().get(0).getFile().getName());
	}

	@Test
	public void testGeneratedInputDoNotCreateAtStart() throws Exception {
		TransMeta meta = createTransMeta("generated-input-do-not-create-at-start.ktr", "number_rows", "10");
		Trans trans = new Trans(meta);

		trans.execute(new String[0]);
		trans.waitUntilFinished();

		assertEquals("Expected one result file", 1, trans.getResult().getResultFilesList().size());
		System.out.println(trans.getResult().getResultFilesList().get(0).getFile().getName());
	}

	@Test
	public void testEmptyGeneratedInput() throws Exception {
		TransMeta meta = createTransMeta("generated-input-do-not-create-at-start.ktr", "number_rows", "0");
		Trans trans = new Trans(meta);

		trans.execute(new String[0]);
		trans.waitUntilFinished();

		assertEquals("Expected no result file", 0, trans.getResult().getResultFilesList().size());
	}

	@Test
	public void testCompressedInputCreateAtStart() throws Exception {
		File csv = getCsvFile("test.csv.gz");
		TransMeta meta = createTransMeta("gzip-input-create-at-start.ktr", "input_file", csv.getAbsolutePath());
		Trans trans = new Trans(meta);

		trans.execute(new String[0]);
		trans.waitUntilFinished();

		assertEquals("Expected one result file", 1, trans.getResult().getResultFilesList().size());
		System.out.println(trans.getResult().getResultFilesList().get(0).getFile().getName());
	}

	@Test
	public void testCompressedInputDoNotCreateAtStart() throws Exception {
		File csv = getCsvFile("test.csv.gz");
		TransMeta meta = createTransMeta("gzip-input-do-not-create-at-start.ktr", "input_file", csv.getAbsolutePath());
		Trans trans = new Trans(meta);

		trans.execute(new String[0]);
		trans.waitUntilFinished();

		assertEquals("Expected one result file", 1, trans.getResult().getResultFilesList().size());
		System.out.println(trans.getResult().getResultFilesList().get(0).getFile().getName());
	}

	@Test
	public void testEmptyCompressedInput() throws Exception {
		File csv = getCsvFile("empty.csv.gz");
		TransMeta meta = createTransMeta("gzip-input-do-not-create-at-start.ktr", "input_file", csv.getAbsolutePath());
		Trans trans = new Trans(meta);

		trans.execute(new String[0]);
		trans.waitUntilFinished();

		assertEquals("Expected no result file", 0, trans.getResult().getResultFilesList().size());
	}

	private TransMeta createTransMeta(String ktrFilename, String parameter, String value)
			throws Exception {
		InputStream ktr = getClass().getClassLoader().getResourceAsStream(ktrFilename);
		TransMeta meta = new TransMeta(ktr, null, true, null, null);
		meta.setParameterValue(parameter, value);
		meta.setLogLevel(LogLevel.DETAILED);
		return meta;
	}

	private File getCsvFile(String filename) throws IOException {
		InputStream input = getClass().getClassLoader().getResourceAsStream(filename);
		File csvFile = tempFolder.newFile();
		try (FileOutputStream output = new FileOutputStream(csvFile)) {
			IOUtils.copy(input, output);
		}
		return csvFile;
	}
}
